rollout TimeAndFrameManager "Time and Frame Manager"
(
    label lbl_frameRate "Frame Rate: " align:#center width:250
    label lbl_currentTime "Current Time: " align:#center width:250

    label spacer1 "" height:10

    timer realTimeTimer interval:100 active:true

    group "Jump To Time"
    (
        spinner spn_hours "Hours:" range:[0,10000,0] type:#integer fieldwidth:80 align:#right
        spinner spn_minutes "Minutes:" range:[0,59,0] type:#integer fieldwidth:80 align:#right
        spinner spn_seconds "Seconds:" range:[0,59,0] type:#integer fieldwidth:80 align:#right

        label spacer2 "" height:5

        button btn_jumpTime "Jump To Time" width:250 height:30 align:#center
    )

    label spacer3 "" height:15

    group "Navigation Time"
    (
        button btn_goStart "Go To Start" width:120 align:#left across:2
        button btn_goEnd "Go To End" width:120 align:#right

        label spacer4 "" height:5

        button btn_updateInfo "Update Info" width:250 align:#center
    )

    label spacer5 "" height:15

    group "Frame Manager"
    (
        spinner spn_startFrame "Start Frame:" range:[0,10000,0] type:#integer fieldwidth:60 align:#left across:2
        spinner spn_endFrame "End Frame:" range:[0,1000000,30] type:#integer fieldwidth:60 align:#right

        label spacer6 "" height:5

        edittext edt_middleFrame "Middle Frame:" readonly:true text:"15" width:250 align:#center
    )

    label spacer8 "" height:15

    group "Navigation Frame"
    (
        button btn_goStartFrame "Go Start" width:80 align:#left across:3
        button btn_goMiddleFrame "Go Middle" width:80 align:#center
        button btn_goEndFrame "Go End" width:80 align:#right
    )

    label spacer9 "" height:20

    label lbl_author "Created by Elvis Gintsyak" align:#center

    fn formattedTime t =
    (
        local frames = (t as float) / (ticksPerFrame as float)
        local currentFrameRate = frameRate as float
        local secondsTotal = frames / currentFrameRate

        local hours = (floor (secondsTotal / 3600)) as integer
        local minutes = (floor ((secondsTotal - (hours * 3600)) / 60)) as integer
        local seconds = (floor (secondsTotal - (hours * 3600) - (minutes * 60))) as integer
        local framesLeft = (floor ((secondsTotal - floor secondsTotal) * currentFrameRate)) as integer

        return (formattedPrint hours format:"02d") + ":" + (formattedPrint minutes format:"02d") + ":" + (formattedPrint seconds format:"02d") + ":" + (formattedPrint framesLeft format:"02d")
    )

    fn updateLabels =
    (
        local currentFrameRate = frameRate
        lbl_frameRate.text = "Frame Rate: " + (currentFrameRate as string) + " fps"
        lbl_currentTime.text = "Current Time: " + formattedTime (currentTime as float)
    )

    on realTimeTimer tick do
    (
        updateLabels()
    )

    on btn_jumpTime pressed do
    (
        local totalSeconds = (spn_hours.value * 3600) + (spn_minutes.value * 60) + (spn_seconds.value)
        local currentFrameRate = frameRate as float
        local targetFrame = totalSeconds * currentFrameRate
        sliderTime = targetFrame
    )

    on btn_goStart pressed do (sliderTime = animationRange.start)
    on btn_goEnd pressed do (sliderTime = animationRange.end)

    on btn_updateInfo pressed do
    (
        updateLabels()
    )

    on spn_startFrame changed val do
    (
        edt_middleFrame.text = (((spn_startFrame.value + spn_endFrame.value) / 2.0) as integer) as string
    )

    on spn_endFrame changed val do
    (
        edt_middleFrame.text = (((spn_startFrame.value + spn_endFrame.value) / 2.0) as integer) as string
    )

    on btn_goStartFrame pressed do (sliderTime = spn_startFrame.value)
    on btn_goMiddleFrame pressed do
    (
        local middle = (spn_startFrame.value + spn_endFrame.value) / 2.0
        sliderTime = (middle as integer)
    )
    on btn_goEndFrame pressed do (sliderTime = spn_endFrame.value)
)

createDialog TimeAndFrameManager 320 510
